function [Iout1,param1] = DLMRI(data,Q1,DLMRIparams,ct,datafull,nl)

%Function for reconstructing MR image from undersampled raw (complex) k-space data.

%Note that all input parameters need to be set prior to simulation. We provide some example settings for parameters below. However, the user is
%advised to carefully choose optimal values for the parameters depending on the specific data or task at hand.


% Inputs -
%       1. data : MRI raw (complex) k-space data - either fully sampled or under-sampled. If data is undersampled, 
%                 it should be zero-padded at non-sampled matrix locations and input at full matrix size.
%       2. Q1 : Sampling Mask (of same size as data) with zeros at non-sampled locations and ones at sampled locations.
%       3. DLMRIparams: Structure that contains the parameters of the DLMRI algorithm. The various fields are as follows - 
%                   - num: Number of iterations of the DLMRI algorithm (Example: 20-40)
%                   - n: Patch size (i.e., Total # of pixels in square patch) (e.g., n=36, n=64)
%                   - K2: Number of dictionary atoms (e.g., K2=n)
%                   - N: Number of signals used for training (e.g., N=200*K2)
%                   - T0: Sparsity settings of patch (e.g., T0=round((0.2)*n)  )
%                   - nu: Sets the weight \nu in the algorithm
%                   - KSVDopt: If set to 1, K-SVD learning is done with fixed sparsity. For any other setting, K-SVD learning is done
%                              employing both sparsity level and an error threshold. (e.g., KSVDopt=2 is recommended)
%                   - thr: Thresholds used in sparse representation of patches after training and possibly during training. 
%                          To use the thresholds during training, set KSVDopt appropriately (recommended). 
%                          `thr' must be a vector of same size as the number (num) of DLMRI iterations. Each element in the vector is 
%                          the threshold for the corresponding iteration number. (e.g., thr=(0.023)*[2 2 2 2 1.4*ones(1,num-4)]  ).
%                          Threshold vector must be set carefully in general to ensure monotonic and maximum improvement in 
%                          reconstruction metrics (e.g., PSNR) over the chosen num iterations.
%                   - numiterateKSVD: Number of iterations within the K-SVD algorithm. (e.g., numiterateKSVD = 15, 20, 10)
%                   - r: Overlap Stride (e.g., r=1)
%        4. ct : If set to 1, the code additionally outputs various performance metrics computed over the algorithm iterations. Otherwise, set to 0.
%        5. datafull : Fully sampled raw MRI k-space data. Required whenever input parameter `ct' is set to 1.
%        6. nl : If set to 1, it indicates that the input data is normalized (i.e., the peak intensity value in the underlying image is approximately 1). 
%                For any other value of `nl', the code automatically applies a normalization before the algorithm begins.
%
%    K-SVD algorithm is initialized as mentioned in our IEEE TMI paper.
%
% Outputs -
%       1. Iout1 - Image reconstructed with DLMRI algorithm from undersampled data - only magnitude is output.
%       2. param1 - Structure containing various performance metrics, etc. for DLMRI algorithm.
%                 - Dictionary : Final DLMRI dictionary
%                 - PSNR0 : PSNR of zero-filled reconstruction (output only when ct is set to 1)
%                 - PSNR : PSNR of the reconstruction at each iteration of the DLMRI algorithm (output only when ct is set to 1)
%                 - HFEN : HFEN of the reconstruction at each iteration of the DLMRI algorithm (output only when ct is set to 1)
%                 - itererror : norm of the difference between the reconstructions at successive iterations

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DLMRI Parameters initialization

num=DLMRIparams.num;  %DLMRI algorithm iteration count.
%The value of `num' in general depends on how good the initial zero-filling
%reconstruction is. Better initial reconstruction typically implies a smaller value of num.

La2=DLMRIparams.nu; % \nu weighting of paper - can be computed using measurement noise level/standard deviation if known.
%If La2 is very large (La2>1e20), then the sampled k-space locations are filled back without averaging in the reconstruction update step 
%of our algorithm.

r=DLMRIparams.r; %Overlap Stride
n=DLMRIparams.n; %patch size
K2=DLMRIparams.K2; %number of dictionary atoms
N=DLMRIparams.N; %number of training signals
T0=DLMRIparams.T0; %sparsity levels
op=DLMRIparams.KSVDopt; %Type of K-SVD learning
threshi=DLMRIparams.thr; %error threshold for patches - allows squared error of (threshi^2)*n per patch during sparse coding (see further in code).
numiterKSVD=DLMRIparams.numiterateKSVD; %number of K-SVD iterations


%other parameters of K-SVD algorithm
param.errorFlag=0;
param.L=T0;
param.K=K2;
param.numIteration=numiterKSVD;
param.preserveDCAtom=0;
param.InitializationMethod='GivenMatrix';param.displayProgress=0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%MAIN CODE%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

I5=data;
[aa,bb]=size(data);
index=find(Q1==1);
I2=(double(I5)).*(Q1); %multiply data with sampling mask (doesn't do anything if data is already undersampled)
I11=fftshift(ifft2(fftshift(I2))); % Inverse FFT - gives zero-filled result

if(nl~=1)
sc = (max(max((I11))));  %peak intensity estimated from zero-filled result
I5=I5/sc; I11=I11/sc; %image, k-space normalization
end

if(ct==1)
I1 = fftshift(ifft2(fftshift(datafull))); %fully sampled reconstruction
if(nl~=1)
I1=I1/sc;   %fully sampled image normalization
end
PSNR0=20*log10(sqrt(aa*bb)/norm(double(abs(I11))-double(abs(I1)),'fro')); %PSNR of zero-filled reconstruction (with respect to fully sampled reconstruction)
param1.PSNR0=PSNR0;
end

%initializing simulation metrics
itererror=zeros(num,1);
if(ct==1)
highfritererror=zeros(num,1);PSNR1=zeros(num,1);
end

%DLMRI iterations
for kp=1:num
    
    Iiter=I11;     
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %Creating image patches
    
    [blocks,idx] = my_im2col(I11,[sqrt(n),sqrt(n)],r); br=mean(blocks); %image patches
    TE=blocks-(ones(n,1)*br);  %subtracting means of patches
    [rows,cols] = ind2sub(size(I11)-sqrt(n)+1,idx);
    N2=size(blocks,2); %total number of overlapping image patches
    
    de=randperm(N2);
    
    %Check if specified number of training signals is less or greater than the available number of patches.
    if(N2>N)
        N4=N;
    else
        N4=N2;
    end
    
    YH=TE(:,de(1:N4));  %Training data - using random subset of all patches
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %dictionary initialization : PCA + random training patches
    
    [UH,SH,VH]=svd(YH*YH');
    D0=zeros(n,K2);
    [hh,jj]=size(UH);
    if(K2>=n)
        D0(:,1:jj)=UH;
        p1=randperm(N4);
        for py=jj+1:K2
            D0(:,py)=YH(:,p1(py-jj));
        end
    else
        D0(:,1:K2)=UH(:,1:K2);
    end
    param.initialDictionary=D0; %initial dictionary for K-SVD algorithm
    YHU=YH;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %K-SVD algorithm  - two versions below
    if(op==1)
        [DT2,output] = KSVDC(YHU,param);  %K-SVD with sparsity threshold only
    else
        [DT2,output] = KSVDC2(YHU,param,threshi(kp)); %K-SVD with both sparsity and error threshold (faster,better)
    end
    
    %DLerror=(norm(YHU -(DT2*output.CoefMatrix),'fro'))^2;  %dictionary fitting error - uncomment to monitor
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %Computing sparse representations of all patches and summing up the patch approximations
    
    D=DT2;
    Weight= zeros(aa,bb);IMout = zeros(aa,bb); bbb=sqrt(n);
    for jj = 1:10000:size(blocks,2)
        jumpSize = min(jj+10000-1,size(blocks,2)); ZZ=TE(:,jj:jumpSize);
        Coefs = OMPerrn(D,ZZ,threshi(kp),2*param.L);  %sparse coding of patches
        ZZ= D*Coefs + (ones(size(blocks,1),1) * br(jj:jumpSize)); %sparse approximations of patches
        
        %summing up patch approximations
        for i  = jj:jumpSize
            col = cols(i); row = rows(i);
            block =reshape(ZZ(:,i-jj+1),[bbb,bbb]);
            IMout(row:row+bbb-1,col:col+bbb-1)=IMout(row:row+bbb-1,col:col+bbb-1)+block;
            Weight(row:row+bbb-1,col:col+bbb-1)=Weight(row:row+bbb-1,col:col+bbb-1)+ones(bbb);
        end;
    end
    
    I3n=IMout./Weight; %patch-averaged result
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    I2=ifftshift(fft2(ifftshift(I3n)));    %Move from image domain to k-space
    
    %K-space update formula
    if(La2>1e20)
        I2(index)=I5(index);
    else
        
        I2(index)= (1/(1+(La2)))*(I2(index) + (La2)*I5(index));
    end
    
    I11=fftshift(ifft2(fftshift(I2))); %Use Inverse FFT to get back to image domain
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %Compute various performance metrics
    
    itererror(kp)= norm(abs(Iiter) - abs(I11),'fro');
    if(ct==1)
    highfritererror(kp)=norm(imfilter(abs(I11),fspecial('log',15,1.5)) - imfilter(abs(I1),fspecial('log',15,1.5)),'fro');
    PSNR1(kp)=20*log10(sqrt(aa*bb)/norm(double(abs(I11))-double(abs(I1)),'fro'));
    end
        
end

%Outputs
Iout1=abs(I11); %User should remove "abs" here if both magnitude and phase parts of reconstruction are desired.
if(ct==1)
param1.PSNR=PSNR1;
param1.HFEN=highfritererror;
end
param1.itererror=itererror;
param1.Dictionary=D;
