function [Iout1,param1] = DLMRIDemoComplex(I1,Q1,sigmai,sigma,DLMRIparams)

%This function takes a Complex valued MR image as input, and then tries to reconstruct it from simulated noisy undersampled k-space data.

%Note that all input parameters need to be set prior to simulation. We provide some example settings for the input parameters below. However, the user is
%advised to carefully choose optimal values for the parameters depending on the specific data or task at hand.


% Inputs -
%       1. I1 : Input Complex MR Image.
%       2. Q1 : Sampling Mask for 2D DFT k-space (Center of mask is center of k-space) with zeros at non-sampled locations and ones at sampled locations.
%       3. sigmai : Noise level (standard deviation of complex noise) in the DFT space of the peak-normalized input image.
%                   To be set to 0 if input image data is assumed noiseless.
%       4. sigma : Simulated noise level (standard deviation of simulated complex k-space noise, if any). To be set to 0 if no new noise is to be added.
%       5. DLMRIparams: Structure that contains the parameters of the DLMRI algorithm. The various fields are as follows - 
%                   - num: Number of iterations of the DLMRI algorithm (Example: 20-100)
%                   - n: Patch size (i.e., Total # of pixels in square patch) (e.g.,  n=36, n=64)
%                   - K2: Number of dictionary atoms (e.g., K2=n)
%                   - N: Number of signals used for training (e.g., N=200*K2)
%                   - T0: Sparsity settings of patch (e.g., T0=round((0.2)*n) )
%                   - Lambda: Sets the weight \nu in the algorithm (e.g., Lambda=140, 280)
%                   - KSVDopt: If set to 1, K-SVD learning is done with fixed sparsity. For any other setting, K-SVD learning is done
%                              employing both sparsity level and an error threshold. (e.g., KSVDopt=2 is recommended)
%                   - thr: Thresholds used in sparse representation of patches during and after training. Must be a vector of 
%                          same size as the number (num) of DLMRI iterations. Each element in the 
%                          vector is the threshold for the corresponding iteration number. (e.g., thr=(0.023)*[2 2 2 2 1.4*ones(1,num-4)]  ).
%                          Threshold vector must be set carefully in general to ensure monotonic and maximum improvement in 
%                          reconstruction metrics (e.g., PSNR) over the chosen num iterations.
%                   - numiterateKSVD: Number of iterations within the K-SVD algorithm. (e.g., numiterateKSVD = 20)
%                   - r: Overlap Stride (e.g., r=1)
%
%    K-SVD algorithm is initialized as mentioned in our IEEE TMI paper.
%
% Outputs -
%       1. Iout1 - Image reconstructed with DLMRI algorithm from undersampled data - only magnitude is output.
%       2. param1 - Structure containing various performance metrics, etc. from simulation for DLMRI.
%                 - InputPSNR : PSNR of fully sampled noisy reconstruction after adding simulated noise.
%                 - PSNR0 : PSNR of zero-filled reconstruction
%                 - PSNR : PSNR of the reconstruction at each iteration of the DLMRI algorithm
%                 - HFEN : HFEN of the reconstruction at each iteration of the DLMRI algorithm
%                 - itererror : norm of the difference between the reconstructions at successive iterations
%                 - Dictionary : Final DLMRI dictionary
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%DLMRI Parameter initialization

sigma2=sqrt((sigmai^2)+(sigma^2));  %Effective k-space noise level 

Lambda=DLMRIparams.Lambda; %Lambda parameter

DLMRIparams.nu = (Lambda)/(sigma2); % \nu weighting of paper

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%MAIN SIMULATION CODE%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

I1=double(I1(:,:,1));
I1=I1/(max(max((abs(I1))))); %Normalize input image
[aa,bb]=size(I1);            %Compute size of image

DZ=((sigma/sqrt(2))*(randn(aa,bb)+(0+1i)*randn(aa,bb)));  %simulating noise

I5 = ifftshift(fft2(ifftshift(I1))); %k-space of input image
I5=I5+DZ;             %add simulated noise in k-space

%Compute PSNR (with respect to input image) after adding simulated noise
IG=fftshift(ifft2(fftshift(I5)));
InputPSNR=20*log10((sqrt(aa*bb))/norm(double(abs(IG))-double(abs(I1)),'fro'));

I2=(double(I5)).*(Q1);  %Apply sampling mask in k-space

[Iout1,param1] = DLMRI(I2,Q1,DLMRIparams,1,double(I5),1);   %Call to DLMRI Code for reconstruction

param1.InputPSNR=InputPSNR;
